#include "in_shp.h"
#include "shp_file.h"

namespace image_loader
{
	IntPtr image_loader::Load_Image(String^ filename, IntPtr ptr, IntPtr size, IntPtr buffer)
	{
		char*   fname = StringToCharstring(filename);
		SHP_File* shp = new SHP_File();
		shp->OpenSHPFile(fname);
		shp->ReadSHPFile();
		shp->CloseSHPFile();
		delete [] fname;

		int     points, parts;
		int    *starts, *partlengths;
		double *x, *y;

		xml_node^ xn = gcnew xml_node( nullptr, "OutputImage", "", "" );

		// BOUNDING BOX
		xn->XmlSetChildValueByName("BoundingBoxLeft",   shp->Box[0].ToString(), false);
		xn->XmlSetChildValueByName("BoundingBoxTop",    shp->Box[1].ToString(), false);
		xn->XmlSetChildValueByName("BoundingBoxRight",  shp->Box[2].ToString(), false);
		xn->XmlSetChildValueByName("BoundingBoxBottom", shp->Box[3].ToString(), false);

		// POINTS
		shp->CollapsePoints(points, x, y);
		xn->XmlSetChildValueByName("PointsCount",           points, false);
		xn->XmlSetChildValueByName("PointsXArray", (int)(void*)(x), false);
		xn->XmlSetChildValueByName("PointsYArray", (int)(void*)(y), false);

		// POLYLINES
		shp->CollapsePolylines(points, parts, starts, partlengths, x, y);
		xn->XmlSetChildValueByName("PolylinesPartCount",       parts,                     false);
		xn->XmlSetChildValueByName("PolylinesPointCount",      points,                    false);
		xn->XmlSetChildValueByName("PolylinesStartArray",      (int)(void*)(starts),      false);
		xn->XmlSetChildValueByName("PolylinesPartLengthArray", (int)(void*)(partlengths), false);
		xn->XmlSetChildValueByName("PolylinesXArray",          (int)(void*)(x),           false);
		xn->XmlSetChildValueByName("PolylinesYArray",          (int)(void*)(y),           false);

		// POLYGONS
		shp->CollapsePolygons(points, parts, starts, partlengths, x, y);
		xn->XmlSetChildValueByName("PolygonsPartCount",        parts,                     false);
		xn->XmlSetChildValueByName("PolygonsPointCount",       points,                    false);
		xn->XmlSetChildValueByName("PolygonsStartArray",       (int)(void*)(starts),      false);
		xn->XmlSetChildValueByName("PolygonsPartLengthArray",  (int)(void*)(partlengths), false);
		xn->XmlSetChildValueByName("PolygonsXArray",           (int)(void*)(x),           false);
		xn->XmlSetChildValueByName("PolygonsYArray",           (int)(void*)(y),           false);

		IntPtr ip = (IntPtr)((void*)(xn->XML_Tree_Marshal()));
		delete shp;
		delete xn;
		return ip;
	}


	IntPtr image_loader::GetImageProperties(String^ filename)
	{
		char*   fname = StringToCharstring(filename);
		SHP_File* shp = new SHP_File();
		shp->OpenSHPFile(fname);
		shp->CloseSHPFile();
		delete [] fname;

		xml_node^ xn = gcnew xml_node( nullptr, "OutputImage", "", "" );

		// BOUNDING BOX
		xn->XmlSetChildValueByName("BoundingBoxLeft",   shp->Box[0].ToString(), false);
		xn->XmlSetChildValueByName("BoundingBoxTop",    shp->Box[1].ToString(), false);
		xn->XmlSetChildValueByName("BoundingBoxRight",  shp->Box[2].ToString(), false);
		xn->XmlSetChildValueByName("BoundingBoxBottom", shp->Box[3].ToString(), false);
		// POINTS
		xn->XmlSetChildValueByName("PointsCount",  0, false);
		xn->XmlSetChildValueByName("PointsXArray", 0, false);
		xn->XmlSetChildValueByName("PointsYArray", 0, false);
		// POLYLINES
		xn->XmlSetChildValueByName("PolylinesPartCount",       0, false);
		xn->XmlSetChildValueByName("PolylinesPointCount",      0, false);
		xn->XmlSetChildValueByName("PolylinesStartArray",      0, false);
		xn->XmlSetChildValueByName("PolylinesPartLengthArray", 0, false);
		xn->XmlSetChildValueByName("PolylinesXArray",          0, false);
		xn->XmlSetChildValueByName("PolylinesYArray",          0, false);
		// POLYGONS
		xn->XmlSetChildValueByName("PolygonsPartCount",        0, false);
		xn->XmlSetChildValueByName("PolygonsPointCount",       0, false);
		xn->XmlSetChildValueByName("PolygonsStartArray",       0, false);
		xn->XmlSetChildValueByName("PolygonsPartLengthArray",  0, false);
		xn->XmlSetChildValueByName("PolygonsXArray",           0, false);
		xn->XmlSetChildValueByName("PolygonsYArray",           0, false);

		IntPtr ip = (IntPtr)((void*)(xn->XML_Tree_Marshal()));
		delete shp;
		delete xn;
		return ip;
	}
}

