#include "shp_file.h"

using namespace std;

#ifndef INT_FORMAT_CONVERT
#define INT_FORMAT_CONVERT(a) ((((a) >> 24) & 0xff) | (((a) >> 8) & 0xff00) | (((a) << 8) & 0xff0000) | (((a) & 0xff) << 24))
#endif


void SHP_File::Convert_SHP_FILE_HEADER(SHP_FILE_HEADER& shp_fh)
{
	shp_fh.FileCode   = INT_FORMAT_CONVERT(shp_fh.FileCode);
	shp_fh.FileLength = INT_FORMAT_CONVERT(shp_fh.FileLength);
}


void SHP_File::Convert_SHP_RECORD_HEADER(SHP_RECORD_HEADER& shp_rh)
{
	shp_rh.RecordNumber  = INT_FORMAT_CONVERT(shp_rh.RecordNumber);
	shp_rh.ContentLength = INT_FORMAT_CONVERT(shp_rh.ContentLength);
}


void SHP_File::OpenSHPFile(char* filename)
{
	errno_t err = fopen_s(&fp, filename, "rb");
	if (err)
	{
		fp = 0;
		return;
	}

	fread_s(&shp_fh, sizeof(SHP_FILE_HEADER), sizeof(SHP_FILE_HEADER), 1, fp);
	Convert_SHP_FILE_HEADER(shp_fh);

	Box[0] = shp_fh.Xmin;
	Box[1] = shp_fh.Ymin;
	Box[2] = shp_fh.Xmax;
	Box[3] = shp_fh.Ymax;
}


void SHP_File::ReadSHPFile()
{
	SHP_MIX* shpmix;
	while ( (shpmix = ReadSHPRecord()) != 0 )
	{
		if (shpmix->ShapeType == SHPTYPE_NullShape) SHP_File::DestroySHPMIX(shpmix);
		else                                        ShapeList.push_back(shpmix);
	}
}


SHP_MIX* SHP_File::ReadSHPRecord()
{
	if (fp == 0) return 0;
	SHP_MIX   *shpmix = 0;
	int        num_points = 0, num_parts = 0;
	int        shape_type;

	size_t len = fread_s(&shp_rh, sizeof(SHP_RECORD_HEADER), sizeof(SHP_RECORD_HEADER), 1, fp);
	if (len <= 0) return 0;

	fread_s(&shape_type, 4, 4, 1, fp);
	Convert_SHP_RECORD_HEADER(shp_rh);
	printf("Record  Number: %d\nContent Length: %d\nShapeType:      %x\n", shp_rh.RecordNumber, shp_rh.ContentLength * 2, shape_type);

	shpmix            = new SHP_MIX;    
	memset(shpmix, 0, sizeof(SHP_MIX));
	shpmix->ShapeType = shape_type;

	if (shape_type == SHPTYPE_NullShape) return shpmix;

	if (shape_type == SHPTYPE_Point || shape_type == SHPTYPE_PointZ || shape_type == SHPTYPE_PointM)
	{
                                          fread_s(&(shpmix->X), 16, 8, 2, fp); // X, Y
		if (shape_type == SHPTYPE_PointZ) fread_s(&(shpmix->Z), 16, 8, 2, fp); // Z, M
		if (shape_type == SHPTYPE_PointM) fread_s(&(shpmix->M),  8, 8, 1, fp); // M
		return shpmix;
	}

	fread_s(shpmix->Box, 32, 8, 4, fp); // BOX
	if ( SHPNEEDPARTS[  shape_type] ) fread_s(&(shpmix->NumParts),  4, 4, 1, fp);
	if ( SHPNEEDPOINTS[ shape_type] ) fread_s(&(shpmix->NumPoints), 4, 4, 1, fp);
	num_parts  = shpmix->NumParts;
	num_points = shpmix->NumPoints;
	printf("\tNumPoints: %d\n\tNumParts:  %d\n", num_points, num_parts);

	switch (shape_type)
	{
	case SHPTYPE_MultiPatch:  // 31
		shpmix->PartTypes = new int[num_parts];
		printf("\tallocate PartTypes...\n");

	case SHPTYPE_PolyLineZ:   // 13
	case SHPTYPE_PolygonZ:    // 15
		shpmix->ZArray = new double[num_points];
		printf("\tallocate ZArray...\n");

	case SHPTYPE_PolyLineM:   // 23
	case SHPTYPE_PolygonM:    // 25
		shpmix->MArray = new double[num_points];
		printf("\tallocate MArray...\n");

	case SHPTYPE_PolyLine:   // 3
	case SHPTYPE_Polygon:    // 5
		shpmix->Parts  = new int[num_parts];
		shpmix->Points = new SHP_Point[num_points];
		printf("\tallocate Parts...\n");
		printf("\tallocate Points...\n");
		break;

	case SHPTYPE_MultiPointZ: // 18
		shpmix->ZArray = new double[num_points];
		printf("\tallocate ZArray...\n");

	case SHPTYPE_MultiPoint: // 8
	case SHPTYPE_MultiPointM: // 28
		shpmix->Points = new SHP_Point[num_points];
		shpmix->MArray = new double[num_points];
		printf("\tallocate Points...\n");
		printf("\tallocate MArray...\n");
		break;
	}

	if (SHPNEEDPARTS[    shape_type]) fread_s(shpmix->Parts,     sizeof(int)       * num_parts,  sizeof(int)       * num_parts,  1, fp);
	if (SHPNEEDPARTTYPES[shape_type]) fread_s(shpmix->PartTypes, sizeof(int)       * num_parts,  sizeof(int)       * num_parts,  1, fp);
	if (SHPNEEDPOINTS[   shape_type]) fread_s(shpmix->Points,    sizeof(SHP_Point) * num_points, sizeof(SHP_Point) * num_points, 1, fp);
	if (SHPNEEDZARRAY[   shape_type]) fread_s(shpmix->ZRange,    sizeof(double)    * 2,          sizeof(double)    * 2,          1, fp);
	if (SHPNEEDZARRAY[   shape_type]) fread_s(shpmix->ZArray,    sizeof(double)    * num_points, sizeof(double)    * num_points, 1, fp);
	if (SHPNEEDMARRAY[   shape_type]) fread_s(shpmix->MRange,    sizeof(double)    * 2,          sizeof(double)    * 2,          1, fp);
	if (SHPNEEDMARRAY[   shape_type]) fread_s(shpmix->MArray,    sizeof(double)    * num_points, sizeof(double)    * num_points, 1, fp);

	printf("\n");
	return shpmix;
}


void SHP_File::DestroySHPMIX(SHP_MIX* s)
{
	if (s->Parts)     { delete [] s->Parts;     s->Parts     = 0; }
	if (s->PartTypes) { delete [] s->PartTypes; s->PartTypes = 0; }
	if (s->Points)    { delete [] s->Points;    s->Points    = 0; }
	if (s->MArray)    { delete [] s->MArray;    s->MArray    = 0; }
	if (s->ZArray)    { delete [] s->ZArray;    s->ZArray    = 0; }
	delete s;
}



void SHP_File::CollapsePoints(int& points, double*& xarray, double*& yarray)
{
	points = 0;
	xarray = yarray = 0;
	std::list<SHP_MIX*>::iterator it;
	
	for (it = ShapeList.begin(); it != ShapeList.end(); it++)
	{
		SHP_MIX* sm = *it;

		// SHPTYPE_Point         1
		// SHPTYPE_MultiPoint    8
		// SHPTYPE_PointZ       11
		// SHPTYPE_MultiPointZ  18
		// SHPTYPE_PointM       21
		// SHPTYPE_MultiPointM  28

		int shape_type = sm->ShapeType;
		if (shape_type == SHPTYPE_Point || shape_type == SHPTYPE_PointZ || shape_type == SHPTYPE_PointM)
		{
			points += 1;
		}
		else if (sm->ShapeType == SHPTYPE_MultiPoint || sm->ShapeType == SHPTYPE_MultiPointZ || sm->ShapeType == SHPTYPE_MultiPointM )
		{
			points += sm->NumPoints;
		}
	}

	if (!points) return;

	int offs = 0;
	xarray = new double[points];
	yarray = new double[points];

	for (it = ShapeList.begin(); it != ShapeList.end(); it++)
	{
		SHP_MIX* sm = *it;

		int shape_type = sm->ShapeType;
		if (shape_type == SHPTYPE_Point || shape_type == SHPTYPE_PointZ || shape_type == SHPTYPE_PointM)
		{
			xarray[offs] = sm->X;
			yarray[offs] = sm->Y;
			offs++;
		}
		else if (sm->ShapeType == SHPTYPE_MultiPoint || sm->ShapeType == SHPTYPE_MultiPointZ || sm->ShapeType == SHPTYPE_MultiPointM )
		{
			int cnt = sm->NumPoints;
			for (int i = 0; i < cnt; i++)
			{
				xarray[offs] = sm->Points[i].X;
				yarray[offs] = sm->Points[i].Y;
				offs++;
			}
		}
	}
}



void SHP_File::CollapsePolygons(int& points, int& parts, int*& starts, int*& partlengths, double*& xarray, double*& yarray)
{
	CollapsePolyX(points, parts, starts, partlengths, xarray, yarray, 2);
}

void SHP_File::CollapsePolylines(int& points, int& parts, int*& starts, int*& partlengths, double*& xarray, double*& yarray)
{
	CollapsePolyX(points, parts, starts, partlengths, xarray, yarray, 0);
}


void SHP_File::CollapsePolyX(int& points, int& parts, int*& starts, int*& partlengths, double*& xarray, double*& yarray, int type)
{
	points = parts = 0;
	xarray = yarray = 0;
	starts = partlengths = 0;
	std::list<SHP_MIX*>::iterator it;

	//System::Windows::Forms::MessageBox::Show("ShapeCount: " + ShapeList.size());

	for (it = ShapeList.begin(); it != ShapeList.end(); it++)
	{
		SHP_MIX* sm = *it;

		// SHPTYPE_Polyline     3 + type (0/2)  5 = SHPTYPE_Polygon
		// SHPTYPE_PolylineZ   13 + type (0/2) 15 = SHPTYPE_PolygonZ
		// SHPTYPE_PolylineM   23 + type (0/2) 25 = SHPTYPE_PolygonM

		if (sm->ShapeType == (3 + type) || sm->ShapeType == (13 + type) || sm->ShapeType == (23 + type))
		{
			points += sm->NumPoints;
			parts += sm->NumParts;
		}
	}

	if (!points) return;

	starts      = new int[parts];
	partlengths = new int[parts];
	xarray      = new double[points];
	yarray      = new double[points];

	int offs = 0, poffs = 0, soffs = 0;

	for (it = ShapeList.begin(); it != ShapeList.end(); it++)
	{
		SHP_MIX* sm = *it;

		if (sm->ShapeType == 3 + type || sm->ShapeType == 13 + type || sm->ShapeType == 23 + type)
		{
			int cnt = sm->NumPoints;
			for (int i = 0; i < cnt; i++)
			{
				xarray[offs] = sm->Points[i].X;
				yarray[offs] = sm->Points[i].Y;
				offs++;
			}			

			cnt = sm->NumParts;
			for (int i = 0; i < cnt; i++) starts[poffs++] = sm->Parts[i] + soffs;
			soffs += sm->NumPoints;
		}
	}

	for (int i = 0; i < parts - 1; i++) partlengths[i] = starts[i + 1] - starts[i];
	partlengths[parts - 1] = points - starts[parts - 1];
}

