#ifndef SHP_TYPES
#define SHP_TYPES

#pragma pack(push,1)

// ---------------------------------------------------------

#define SHPTYPE_NullShape     0
#define SHPTYPE_Point         1
#define SHPTYPE_PolyLine      3
#define SHPTYPE_Polygon       5
#define SHPTYPE_MultiPoint    8
#define SHPTYPE_PointZ       11
#define SHPTYPE_PolyLineZ    13
#define SHPTYPE_PolygonZ     15
#define SHPTYPE_MultiPointZ  18
#define SHPTYPE_PointM       21
#define SHPTYPE_PolyLineM    23
#define SHPTYPE_PolygonM     25
#define SHPTYPE_MultiPointM  28
#define SHPTYPE_MultiPatch   31

// ---------------------------------------------------------

struct SHP_FILE_HEADER
{
	int    FileCode;     // BigEndian    - = 9994
	int    Unused[5];    // BigEndian    - = 0, 0, 0, 0, 0
	int    FileLength;   // BigEndian    - in 2 bytes
	int    Version;      // LittleEndian - = 1000
	int    ShapeType;    // LittleEndian
	double Xmin, Ymin, Xmax, Ymax, Zmin, Zmax, Mmin, Mmax; // LittleEndian - Z/M unused if 0.0
};

struct SHP_RECORD_HEADER { int RecordNumber, ContentLength; }; // BigEndian

// ---------------------------------------------------------

typedef struct { double X, Y; } SHP_Point;

typedef struct
{
	int         ShapeType;
	double      Box[4];     // Bounding Box
	int         NumParts;   // Number of Parts
	int         NumPoints;  // Total Number of Points
	double      X, Y, Z, M; // Potential X, Y, Z, M coordinates for SHP_Point[Z/M]
	int        *Parts;      // Index to First Point in Part
	int        *PartTypes;  // Part Type
	SHP_Point  *Points;     // Points for All Parts
	double      ZRange[2];  // Bounding Z Range
	double     *ZArray;     // Z Values for All Points
	double      MRange[2];  // Bounding Measure Range
	double     *MArray;     // Measures
} SHP_MIX;

// ---------------------------------------------------------

const int SHPPREREADSIZE[32] = { 0, 16,  0, 40,  0, 40,  0,  0, 36,  0,  0, \
                                    32,  0, 40,  0, 40,  0,  0, 36,  0,  0, \
                                    24,  0, 40,  0, 40,  0,  0, 36,  0,  0, 40 };

//                                 0  1  2  3  4  5  6  7  8  9  0  1  2  3  4  5  6  7  8  9  0  1  2  3  4  5  6  7  8  9  0  1
const int SHPNEEDPARTS[32]     = { 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1 };
const int SHPNEEDPARTTYPES[32] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 };
const int SHPNEEDPOINTS[32]    = { 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1 };
const int SHPNEEDMARRAY[32]    = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1 };
const int SHPNEEDZARRAY[32]    = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 };

// ---------------------------------------------------------

#pragma pack(pop) 

#endif
